@@ -1,6 +1,6 @@
 /* -*- mode: C++; c-basic-offset: 4; tab-width: 4 -*-
  *
- * Copyright (c) 2004-2005 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2004-2009 Apple Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -24,13 +24,27 @@
 
 #include <stddef.h>
 #include <string.h>
+#include <malloc/malloc.h>
+
 #include <crt_externs.h>
+#include <Availability.h>
 
 #include "mach-o/dyld.h"
 #include "mach-o/dyld_priv.h"
 
 #include "dyldLock.h"
 
+extern "C" int __cxa_atexit(void (*func)(void *), void *arg, void *dso);
+
+#define DYLD_SHARED_CACHE_SUPPORT (__ppc__ || __i386__ || __ppc64__ || __x86_64__)
+
+// deprecated APIs are still availble on Mac OS X, but not on iPhone OS
+#if __IPHONE_OS_VERSION_MIN_REQUIRED	
+	#define DEPRECATED_APIS_SUPPORTED 0
+#else
+	#define DEPRECATED_APIS_SUPPORTED 1
+#endif
+
 /*
  * names_match() takes an install_name from an LC_LOAD_DYLIB command and a
  * libraryName (which is -lx or -framework Foo argument passed to the static
@@ -84,10 +98,12 @@ const char* libraryName)
 	return(FALSE);
 }
 
+#if DEPRECATED_APIS_SUPPORTED
+
 void NSInstallLinkEditErrorHandlers(
 const NSLinkEditErrorHandlers* handlers)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(
 	void     (*undefined)(const char* symbol_name),
 	NSModule (*multiple)(NSSymbol s, NSModule old, NSModule newhandler),
@@ -103,7 +119,7 @@ const char*
 NSNameOfModule(
 NSModule module)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static const char*  (*p)(NSModule module) = NULL;
 
 	if(p == NULL)
@@ -115,7 +131,7 @@ const char*
 NSLibraryNameForModule(
 NSModule module)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static const char*  (*p)(NSModule module) = NULL;
 
 	if(p == NULL)
@@ -127,7 +143,7 @@ bool
 NSIsSymbolNameDefined(
 const char* symbolName)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static bool (*p)(const char* symbolName) = NULL;
 
 	if(p == NULL)
@@ -140,7 +156,7 @@ NSIsSymbolNameDefinedWithHint(
 const char* symbolName,
 const char* libraryNameHint)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static bool (*p)(const char* symbolName,
 			  const char* libraryNameHint) = NULL;
 
@@ -154,7 +170,7 @@ NSIsSymbolNameDefinedInImage(
 const struct mach_header *image,
 const char* symbolName)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static bool (*p)(const struct mach_header *image,
 			  const char* symbolName) = NULL;
 
@@ -167,7 +183,7 @@ NSSymbol
 NSLookupAndBindSymbol(
 const char* symbolName)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static NSSymbol (*p)(const char* symbolName) = NULL;
 
 	if(p == NULL)
@@ -180,7 +196,7 @@ NSLookupAndBindSymbolWithHint(
 const char* symbolName,
 const char* libraryNameHint)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static NSSymbol (*p)(const char* symbolName,
 			 const char* libraryNameHint) = NULL;
 
@@ -194,7 +210,7 @@ NSLookupSymbolInModule(
 NSModule module,
 const char* symbolName)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static NSSymbol (*p)(NSModule module, const char* symbolName) = NULL;
 
 	if(p == NULL)
@@ -208,7 +224,7 @@ const struct mach_header *image,
 const char* symbolName,
 uint32_t options)
 {
- 	DYLD_READER_LOCK_THIS_BLOCK;
+ 	DYLD_LOCK_THIS_BLOCK;
    static NSSymbol (*p)(const struct mach_header *image,
 			 const char* symbolName,
 			 uint32_t options) = NULL;
@@ -222,7 +238,7 @@ const char*
 NSNameOfSymbol(
 NSSymbol symbol)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static char * (*p)(NSSymbol symbol) = NULL;
 
 	if(p == NULL)
@@ -234,7 +250,7 @@ void *
 NSAddressOfSymbol(
 NSSymbol symbol)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void * (*p)(NSSymbol symbol) = NULL;
 
 	if(p == NULL)
@@ -246,7 +262,7 @@ NSModule
 NSModuleForSymbol(
 NSSymbol symbol)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static NSModule (*p)(NSSymbol symbol) = NULL;
 
 	if(p == NULL)
@@ -258,7 +274,7 @@ bool
 NSAddLibrary(
 const char* pathName)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static bool (*p)(const char* pathName) = NULL;
 
 	if(p == NULL)
@@ -270,7 +286,7 @@ bool
 NSAddLibraryWithSearching(
 const char* pathName)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static bool (*p)(const char* pathName) = NULL;
 
 	if(p == NULL)
@@ -283,7 +299,7 @@ NSAddImage(
 const char* image_name,
 uint32_t options)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static const struct mach_header * (*p)(const char* image_name,
 					   uint32_t options) = NULL;
 
@@ -291,6 +307,7 @@ uint32_t options)
 	    _dyld_func_lookup("__dyld_NSAddImage", (void**)&p);
 	return(p(image_name, options));
 }
+#endif // DEPRECATED_APIS_SUPPORTED
 
 /*
  * This routine returns the current version of the named shared library the
@@ -311,30 +328,30 @@ const char* libraryName)
     struct load_command *load_commands, *lc;
     struct dylib_command *dl;
     char *install_name;
-#ifndef __OPENSTEP__
+#if __LP64__
+    static struct mach_header_64 *mh = NULL;
+#else
     static struct mach_header *mh = NULL;
+#endif
 	if(mh == NULL)
 	    mh = _NSGetMachExecuteHeader();
-#else /* defined(__OPENSTEP__) */
-#ifdef __DYNAMIC__
-    static struct mach_header *mh = NULL;
-	if(mh == NULL)
-	    _dyld_lookup_and_bind("__mh_execute_header", &mh, NULL);
-#else
-    struct mach_header *mh;
-	mh = (struct mach_header *)&_mh_execute_header;
-#endif
-#endif /* __OPENSTEP__ */
 	load_commands = (struct load_command *)
-			((char *)mh + sizeof(struct mach_header));
+#if __LP64__
+			    ((char *)mh + sizeof(struct mach_header_64));
+#else
+			    ((char *)mh + sizeof(struct mach_header));
+#endif
 	lc = load_commands;
 	for(i = 0; i < mh->ncmds; i++){
-	    if(lc->cmd == LC_LOAD_DYLIB){
-		dl = (struct dylib_command *)lc;
-		install_name = (char *)dl + dl->dylib.name.offset;
-		if(names_match(install_name, libraryName) == TRUE)
-		    return(dl->dylib.current_version);
-	    }
+		switch ( lc->cmd ) { 
+			case LC_LOAD_DYLIB:
+			case LC_LOAD_WEAK_DYLIB:
+				dl = (struct dylib_command *)lc;
+				install_name = (char *)dl + dl->dylib.name.offset;
+				if(names_match(install_name, libraryName) == TRUE)
+					return(dl->dylib.current_version);
+				break;
+		}
 	    lc = (struct load_command *)((char *)lc + lc->cmdsize);
 	}
 	return(-1);
@@ -363,7 +380,11 @@ const char* libraryName)
 	    if(mh->filetype != MH_DYLIB)
 		continue;
 	    load_commands = (struct load_command *)
+#if __LP64__
+			    ((char *)mh + sizeof(struct mach_header_64));
+#else
 			    ((char *)mh + sizeof(struct mach_header));
+#endif
 	    lc = load_commands;
 	    for(j = 0; j < mh->ncmds; j++){
 		if(lc->cmd == LC_ID_DYLIB){
@@ -378,6 +399,7 @@ const char* libraryName)
 	return(-1);
 }
 
+#if DEPRECATED_APIS_SUPPORTED
 /*
  * NSCreateObjectFileImageFromFile() creates an NSObjectFileImage for the
  * specified file name if the file is a correct Mach-O file that can be loaded
@@ -390,7 +412,7 @@ NSCreateObjectFileImageFromFile(
 const char* pathName,
 NSObjectFileImage *objectFileImage)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static NSObjectFileImageReturnCode (*p)(const char*, NSObjectFileImage*) = NULL;
 
 	if(p == NULL)
@@ -412,7 +434,7 @@ const void* address,
 size_t size, 
 NSObjectFileImage *objectFileImage)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static NSObjectFileImageReturnCode (*p)(const void*, size_t, NSObjectFileImage*) = NULL;
 
 	if(p == NULL)
@@ -420,6 +442,7 @@ NSObjectFileImage *objectFileImage)
 	return p(address, size, objectFileImage);
 }
 
+#if OBSOLETE_DYLD_API
 /*
  * NSCreateCoreFileImageFromFile() creates an NSObjectFileImage for the 
  * specified core file name if the file is a correct Mach-O core file.
@@ -431,19 +454,20 @@ NSCreateCoreFileImageFromFile(
 const char* pathName,
 NSObjectFileImage *objectFileImage)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static NSObjectFileImageReturnCode (*p)(const char*, NSObjectFileImage*) = NULL;
 
 	if(p == NULL)
 	    _dyld_func_lookup("__dyld_NSCreateCoreFileImageFromFile", (void**)&p);
 	return p(pathName, objectFileImage);
 }
+#endif
 
 bool
 NSDestroyObjectFileImage(
 NSObjectFileImage objectFileImage)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static bool (*p)(NSObjectFileImage) = NULL;
 
 	if(p == NULL)
@@ -458,7 +482,7 @@ NSObjectFileImage objectFileImage,
 const char* moduleName,
 uint32_t options)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static NSModule (*p)(NSObjectFileImage, const char*, unsigned long) = NULL;
 
 	if(p == NULL)
@@ -468,31 +492,6 @@ uint32_t options)
 }
 
 
-/*
- * NSFindSectionAndOffsetInObjectFileImage() takes the specified imageOffset
- * into the specified ObjectFileImage and returns the segment/section name and
- * offset into that section of that imageOffset.  Returns FALSE if the
- * imageOffset is not in any section.  You can used the resulting sectionOffset
- * to index into the data returned by NSGetSectionDataInObjectFileImage.
- * 
- * SPI: currently only used by ZeroLink to detect +load methods
- */
-bool 
-NSFindSectionAndOffsetInObjectFileImage(
-NSObjectFileImage objectFileImage, 
-unsigned long imageOffset,
-const char** segmentName, 	/* can be NULL */
-const char** sectionName, 	/* can be NULL */
-unsigned long* sectionOffset)	/* can be NULL */
-{
-	DYLD_READER_LOCK_THIS_BLOCK;
-    static bool (*p)(NSObjectFileImage, unsigned long, const char**, const char**, unsigned long*) = NULL;
-
-	if(p == NULL)
-	    _dyld_func_lookup("__dyld_NSFindSectionAndOffsetInObjectFileImage", (void**)&p);
-		
-	return p(objectFileImage, imageOffset, segmentName, sectionName, sectionOffset);
-}
 
 
 /*
@@ -503,7 +502,7 @@ uint32_t
 NSSymbolDefinitionCountInObjectFileImage(
 NSObjectFileImage objectFileImage)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static unsigned long (*p)(NSObjectFileImage) = NULL;
 
 	if(p == NULL)
@@ -523,7 +522,7 @@ NSSymbolDefinitionNameInObjectFileImage(
 NSObjectFileImage objectFileImage,
 uint32_t ordinal)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static const char*  (*p)(NSObjectFileImage, uint32_t) = NULL;
 
 	if(p == NULL)
@@ -540,7 +539,7 @@ uint32_t
 NSSymbolReferenceCountInObjectFileImage(
 NSObjectFileImage objectFileImage)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static unsigned long (*p)(NSObjectFileImage) = NULL;
 
 	if(p == NULL)
@@ -561,7 +560,7 @@ NSObjectFileImage objectFileImage,
 uint32_t ordinal,
 bool *tentative_definition) /* can be NULL */
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static const char*  (*p)(NSObjectFileImage, uint32_t, bool*) = NULL;
 
 	if(p == NULL)
@@ -579,7 +578,7 @@ NSIsSymbolDefinedInObjectFileImage(
 NSObjectFileImage objectFileImage,
 const char* symbolName)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static bool (*p)(NSObjectFileImage, const char*) = NULL;
 
 	if(p == NULL)
@@ -602,7 +601,7 @@ const char* segmentName,
 const char* sectionName,
 unsigned long *size) /* can be NULL */
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void* (*p)(NSObjectFileImage, const char*, const char*, unsigned long*) = NULL;
 
 	if(p == NULL)
@@ -611,24 +610,6 @@ unsigned long *size) /* can be NULL */
 	return p(objectFileImage, segmentName, sectionName, size);
 }
 
-/*
- * NSHasModInitObjectFileImage() returns TRUE if the NSObjectFileImage has any
- * module initialization sections and FALSE it it does not.
- *
- * SPI: currently only used by ZeroLink to detect C++ initializers
- */
-bool
-NSHasModInitObjectFileImage(
-NSObjectFileImage objectFileImage)
-{
-	DYLD_READER_LOCK_THIS_BLOCK;
-    static bool (*p)(NSObjectFileImage) = NULL;
-
-	if(p == NULL)
-	    _dyld_func_lookup("__dyld_NSHasModInitObjectFileImage", (void**)&p);
-		
-	return p(objectFileImage);
-}
 
 void
 NSLinkEditError(
@@ -637,7 +618,7 @@ int *errorNumber,
 const char* *fileName,
 const char* *errorString)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(NSLinkEditErrors *c,
 		     int *errorNumber, 
 		     const char* *fileName,
@@ -654,7 +635,7 @@ NSUnLinkModule(
 NSModule module, 
 uint32_t options)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static bool (*p)(NSModule module, uint32_t options) = NULL;
 
 	if(p == NULL)
@@ -663,6 +644,7 @@ uint32_t options)
 	return p(module, options);
 }
 
+#if OBSOLETE_DYLD_API
 NSModule
 NSReplaceModule(
 NSModule moduleToReplace,
@@ -671,6 +653,10 @@ uint32_t options)
 {
 	return(NULL);
 }
+#endif
+
+
+#endif // DEPRECATED_APIS_SUPPORTED
 
 /*
  *_NSGetExecutablePath copies the path of the executable into the buffer and
@@ -686,7 +672,7 @@ _NSGetExecutablePath(
 char *buf,
 uint32_t *bufsize)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static int (*p)(char *buf, uint32_t *bufsize) = NULL;
 
 	if(p == NULL)
@@ -694,13 +680,14 @@ uint32_t *bufsize)
 	return(p(buf, bufsize));
 }
 
+#if DEPRECATED_APIS_SUPPORTED
 void
 _dyld_lookup_and_bind(
 const char* symbol_name,
 void** address,
 NSModule* module)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(const char*, void** , NSModule*) = NULL;
 
 	if(p == NULL)
@@ -715,7 +702,7 @@ const char* library_name_hint,
 void** address,
 NSModule* module)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(const char*, const char*, void**, NSModule*) = NULL;
 
 	if(p == NULL)
@@ -723,19 +710,21 @@ NSModule* module)
 	p(symbol_name, library_name_hint, address, module);
 }
 
+#if OBSOLETE_DYLD_API
 void
 _dyld_lookup_and_bind_objc(
 const char* symbol_name,
 void** address,
 NSModule* module)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(const char* , void**, NSModule*) = NULL;
 
 	if(p == NULL)
 	    _dyld_func_lookup("__dyld_lookup_and_bind_objc", (void**)&p);
 	p(symbol_name, address, module);
 }
+#endif
 
 void
 _dyld_lookup_and_bind_fully(
@@ -743,7 +732,7 @@ const char* symbol_name,
 void** address,
 NSModule* module)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(const char*, void**, NSModule*) = NULL;
 
 	if(p == NULL)
@@ -755,13 +744,14 @@ bool
 _dyld_bind_fully_image_containing_address(
 const void* address)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static bool (*p)(const void*) = NULL;
 
 	if(p == NULL)
 	    _dyld_func_lookup("__dyld_bind_fully_image_containing_address", (void**)&p);
 	return p(address);
 }
+#endif // DEPRECATED_APIS_SUPPORTED
 
 
 /*
@@ -774,7 +764,7 @@ void
 _dyld_register_func_for_add_image(
 void (*func)(const struct mach_header *mh, intptr_t vmaddr_slide))
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(void (*func)(const struct mach_header *mh, intptr_t vmaddr_slide)) = NULL;
 
 	if(p == NULL)
@@ -791,7 +781,7 @@ void
 _dyld_register_func_for_remove_image(
 void (*func)(const struct mach_header *mh, intptr_t vmaddr_slide))
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(void (*func)(const struct mach_header *mh, intptr_t vmaddr_slide)) = NULL;
 
 	if(p == NULL)
@@ -799,6 +789,7 @@ void (*func)(const struct mach_header *mh, intptr_t vmaddr_slide))
 	p(func);
 }
 
+#if OBSOLETE_DYLD_API
 /*
  * _dyld_register_func_for_link_module registers the specified function to be
  * called when a module is bound into the program.  When this function is first
@@ -809,7 +800,7 @@ void
 _dyld_register_func_for_link_module(
 void (*func)(NSModule module))
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(void (*func)(NSModule module)) = NULL;
 
 	if(p == NULL)
@@ -825,7 +816,7 @@ void
 _dyld_register_func_for_unlink_module(
 void (*func)(NSModule module))
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(void (*func)(NSModule module)) = NULL;
 
 	if(p == NULL)
@@ -841,7 +832,7 @@ void
 _dyld_register_func_for_replace_module(
 void (*func)(NSModule oldmodule, NSModule newmodule))
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(void (*func)(NSModule oldmodule,
 				  NSModule newmodule)) = NULL;
 
@@ -862,7 +853,7 @@ NSModule module,
 void **objc_module,
 unsigned long *size)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(NSModule module,
 		     void **objc_module,
 		     unsigned long *size) = NULL;
@@ -878,45 +869,39 @@ unsigned long *size)
  * to be bound.
  */
 void
-_dyld_bind_objc_module(
-const void* objc_module)
+_dyld_bind_objc_module(const void* objc_module)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(const void *objc_module) = NULL;
 
 	if(p == NULL)
 	    _dyld_func_lookup("__dyld_bind_objc_module", (void**)&p);
 	p(objc_module);
 }
+#endif
 
-
-#if __DYNAMIC__
+#if DEPRECATED_APIS_SUPPORTED
 bool
-_dyld_present(
-void)
+_dyld_present(void)
 {
-	// hmmm, this code is in libSystem.dylib, which is loaded by dyld...
+	// this function exists for compatiblity only
 	return true;
 }
 #endif
 
 uint32_t
-_dyld_image_count(
-void)
+_dyld_image_count(void)
 {
 	DYLD_NO_LOCK_THIS_BLOCK;
     static unsigned long (*p)(void) = NULL;
 
-	if(_dyld_present() == 0)
-	    return(0);
 	if(p == NULL)
 	    _dyld_func_lookup("__dyld_image_count", (void**)&p);
 	return(p());
 }
 
 const struct mach_header *
-_dyld_get_image_header(
-uint32_t image_index)
+_dyld_get_image_header(uint32_t image_index)
 {
 	DYLD_NO_LOCK_THIS_BLOCK;
     static struct mach_header * (*p)(uint32_t image_index) = NULL;
@@ -927,8 +912,7 @@ uint32_t image_index)
 }
 
 intptr_t
-_dyld_get_image_vmaddr_slide(
-uint32_t image_index)
+_dyld_get_image_vmaddr_slide(uint32_t image_index)
 {
 	DYLD_NO_LOCK_THIS_BLOCK;
     static unsigned long (*p)(uint32_t image_index) = NULL;
@@ -939,8 +923,7 @@ uint32_t image_index)
 }
 
 const char* 
-_dyld_get_image_name(
-uint32_t image_index)
+_dyld_get_image_name(uint32_t image_index)
 {
 	DYLD_NO_LOCK_THIS_BLOCK;
     static const char*  (*p)(uint32_t image_index) = NULL;
@@ -950,11 +933,22 @@ uint32_t image_index)
 	return(p(image_index));
 }
 
-bool
-_dyld_image_containing_address(
-const void* address)
+// SPI in Mac OS X 10.6
+intptr_t _dyld_get_image_slide(const struct mach_header* mh)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_NO_LOCK_THIS_BLOCK;
+    static intptr_t (*p)(const struct mach_header*) = NULL;
+
+	if(p == NULL)
+	    _dyld_func_lookup("__dyld_get_image_slide", (void**)&p);
+	return(p(mh));
+}
+
+
+bool
+_dyld_image_containing_address(const void* address)
+{
+	DYLD_LOCK_THIS_BLOCK;
     static bool (*p)(const void*) = NULL;
 
 	if(p == NULL)
@@ -966,7 +960,7 @@ const struct mach_header *
 _dyld_get_image_header_containing_address(
 const void* address)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static const struct mach_header * (*p)(const void*) = NULL;
 
 	if(p == NULL)
@@ -977,7 +971,7 @@ const void* address)
 void _dyld_moninit(
 void (*monaddition)(char *lowpc, char *highpc))
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void (*p)(void (*monaddition)(char *lowpc, char *highpc)) = NULL;
 
 	if(p == NULL)
@@ -985,10 +979,10 @@ void (*monaddition)(char *lowpc, char *highpc))
 	p(monaddition);
 }
 
-bool _dyld_launched_prebound(
-void)
+#if DEPRECATED_APIS_SUPPORTED
+bool _dyld_launched_prebound(void)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static bool (*p)(void) = NULL;
 
 	if(p == NULL)
@@ -996,16 +990,16 @@ void)
 	return(p());
 }
 
-bool _dyld_all_twolevel_modules_prebound(
-void)
+bool _dyld_all_twolevel_modules_prebound(void)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static bool (*p)(void) = NULL;
 
 	if(p == NULL)
 	    _dyld_func_lookup("__dyld_all_twolevel_modules_prebound", (void**)&p);
 	return(p());
 }
+#endif // DEPRECATED_APIS_SUPPORTED
 
 
 #include <dlfcn.h>
@@ -1013,11 +1007,13 @@ void)
 #include <pthread.h>
 #include <stdlib.h>
 #include <mach-o/dyld.h>
-#include "dyldLock.h"
-#include "dyldLibSystemThreadHelpers.h"
+#include <servers/bootstrap.h>
+#include "dyldLibSystemInterface.h"
+
 
 // pthread key used to access per-thread dlerror message
 static pthread_key_t dlerrorPerThreadKey;
+static bool dlerrorPerThreadKeyInitialized = false;
 
 // data kept per-thread
 struct dlerrorPerThreadData
@@ -1029,6 +1025,13 @@ struct dlerrorPerThreadData
 // function called by dyld to get buffer to store dlerror message
 static char* getPerThreadBufferFor_dlerror(uint32_t sizeRequired)
 {
+	// ok to create key lazily because this function is called within dyld lock, so there is no race condition
+	if (!dlerrorPerThreadKeyInitialized ) {
+		// create key and tell pthread package to call free() on any data associated with key if thread dies
+		pthread_key_create(&dlerrorPerThreadKey, &free);
+		dlerrorPerThreadKeyInitialized = true;
+	}
+
 	const int size = (sizeRequired < 256) ? 256 : sizeRequired;
 	dlerrorPerThreadData* data = (dlerrorPerThreadData*)pthread_getspecific(dlerrorPerThreadKey);
 	if ( data == NULL ) {
@@ -1049,37 +1052,53 @@ static char* getPerThreadBufferFor_dlerror(uint32_t sizeRequired)
 	return data->message;
 }
 
-// that table passed to dyld containing thread helpers
-static dyld::ThreadingHelpers sThreadHelpers = { 1, &lockForLazyBinding, &unlockForLazyBinding, &getPerThreadBufferFor_dlerror };
+
+#if DYLD_SHARED_CACHE_SUPPORT
+static void shared_cache_missing()
+{
+	// leave until dyld's that might call this are rare
+}
+
+static void shared_cache_out_of_date()
+{
+	// leave until dyld's that might call this are rare
+}
+#endif // DYLD_SHARED_CACHE_SUPPORT
+
+
+// the table passed to dyld containing thread helpers
+static dyld::LibSystemHelpers sHelpers = { 6, &dyldGlobalLockAcquire, &dyldGlobalLockRelease,  
+									&getPerThreadBufferFor_dlerror, &malloc, &free, &__cxa_atexit,
+						#if DYLD_SHARED_CACHE_SUPPORT
+									&shared_cache_missing, &shared_cache_out_of_date,
+						#else
+									NULL, NULL,
+						#endif
+									NULL, NULL,
+									&pthread_key_create, &pthread_setspecific,
+									&malloc_size };
+
 
 //
 // during initialization of libSystem this routine will run
-// and call dyld, registering the threading helpers.
+// and call dyld, registering the helper functions.
 //
-//
-static int registerWithDyld()
+extern "C" void _dyld_initializer() __attribute__((visibility("hidden")));
+void _dyld_initializer()
 {
-    static void (*p)(dyld::ThreadingHelpers*) = NULL;
-
-	// create key and tell pthread package to call free() on any data associated with key if thread dies
-	pthread_key_create(&dlerrorPerThreadKey, &free);
+	DYLD_LOCK_INITIALIZER;
 	
-	if(p == NULL) 
-	    _dyld_func_lookup("__dyld_register_thread_helpers", (void**)&p);
+   void (*p)(dyld::LibSystemHelpers*);
+
+	_dyld_func_lookup("__dyld_register_thread_helpers", (void**)&p);
 	if(p != NULL)
-		p(&sThreadHelpers);
-	
-	return 0;
+		p(&sHelpers);
 }
 
-// should be able to use __attribute__((constructor)) on registerWithDyld, but compiler has bug (3679135)
-// instead use initialization of global to force it to run.
-static int hack = registerWithDyld();
-
 
 char* dlerror()
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static char* (*p)() = NULL;
 
 	if(p == NULL)
@@ -1089,7 +1108,7 @@ char* dlerror()
 
 int dladdr(const void* addr, Dl_info* info)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static int (*p)(const void* , Dl_info*) = NULL;
 
 	if(p == NULL)
@@ -1099,7 +1118,7 @@ int dladdr(const void* addr, Dl_info* info)
 
 int dlclose(void* handle)
 {
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static int (*p)(void* handle) = NULL;
 
 	if(p == NULL)
@@ -1108,18 +1127,36 @@ int dlclose(void* handle)
 }
 
 void* dlopen(const char* path, int mode)
-{
-	DYLD_WRITER_LOCK_THIS_BLOCK;
+{	
+	// dlopen is special. locking is done inside dyld to allow initializer to run without lock
+	DYLD_NO_LOCK_THIS_BLOCK;
+	
     static void* (*p)(const char* path, int) = NULL;
 
 	if(p == NULL)
 	    _dyld_func_lookup("__dyld_dlopen", (void**)&p);
-	return(p(path, mode));
+	void* result = p(path, mode);
+	// use asm block to prevent tail call optimization
+	// this is needed because dlopen uses __builtin_return_address() and depends on this glue being in the frame chain
+	// <rdar://problem/5313172 dlopen() looks too far up stack, can cause crash>
+	__asm__ volatile(""); 
+	
+	return result;
+}
+
+bool dlopen_preflight(const char* path)
+{
+	DYLD_LOCK_THIS_BLOCK;
+    static bool (*p)(const char* path) = NULL;
+
+	if(p == NULL)
+	    _dyld_func_lookup("__dyld_dlopen_preflight", (void**)&p);
+	return(p(path));
 }
 
 void* dlsym(void* handle, const char* symbol)
 {
-	DYLD_READER_LOCK_THIS_BLOCK;
+	DYLD_LOCK_THIS_BLOCK;
     static void* (*p)(void* handle, const char* symbol) = NULL;
 
 	if(p == NULL)
@@ -1127,7 +1164,64 @@ void* dlsym(void* handle, const char* symbol)
 	return(p(handle, symbol));
 }
 
+void dyld_register_image_state_change_handler(dyld_image_states state, 
+											bool batch, dyld_image_state_change_handler handler)
+{
+	DYLD_LOCK_THIS_BLOCK;
+    static void* (*p)(dyld_image_states, bool, dyld_image_state_change_handler) = NULL;
+
+	if(p == NULL)
+	    _dyld_func_lookup("__dyld_dyld_register_image_state_change_handler", (void**)&p);
+	p(state, batch, handler);
+}
 
 
+const struct dyld_all_image_infos* _dyld_get_all_image_infos()
+{
+	DYLD_NO_LOCK_THIS_BLOCK;
+    static struct dyld_all_image_infos* (*p)() = NULL;
+
+	if(p == NULL)
+	    _dyld_func_lookup("__dyld_get_all_image_infos", (void**)&p);
+	return p();
+}
+
+#if !__arm__
+__attribute__((visibility("hidden"))) 
+bool _dyld_find_unwind_sections(void* addr, dyld_unwind_sections* info)
+{
+	DYLD_NO_LOCK_THIS_BLOCK;
+    static void* (*p)(void*, dyld_unwind_sections*) = NULL;
+
+	if(p == NULL)
+	    _dyld_func_lookup("__dyld_find_unwind_sections", (void**)&p);
+	return p(addr, info);
+}
+#endif
+
+
+#if __i386__ || __x86_64__
+__attribute__((visibility("hidden"))) 
+void* _dyld_fast_stub_entry(void* loadercache, long lazyinfo)
+{
+	DYLD_NO_LOCK_THIS_BLOCK;
+    static void* (*p)(void*, long) = NULL;
+
+	if(p == NULL)
+	    _dyld_func_lookup("__dyld_fast_stub_entry", (void**)&p);
+	return p(loadercache, lazyinfo);
+}
+#endif
+
+
+const char* dyld_image_path_containing_address(const void* addr)
+{
+	DYLD_NO_LOCK_THIS_BLOCK;
+    static const char* (*p)(const void*) = NULL;
+
+	if(p == NULL)
+	    _dyld_func_lookup("__dyld_image_path_containing_address", (void**)&p);
+	return p(addr);
+}
 
 
