@@ -21,6 +21,8 @@ sub FAIL
 my $pass_string = shift @ARGV;
 my $fail_string = shift @ARGV;
 
+# redirect stderr to stdout
+open(STDERR, ">/tmp/exit-zero-pass.tmp") || die("$!");
 if(0 == system(@ARGV))
 {
     PASS($pass_string);
@@ -29,5 +31,12 @@ else
 {
     FAIL($fail_string);
 }
+close(STDERR) || die("$!");
+open(OUT, "</tmp/exit-zero-pass.tmp") || die("$!");
+while(<OUT>)
+{
+	print $_;
+}
+close(OUT) || die("$!");
 exit 0;
 
