@@ -27,7 +27,7 @@
 #include "test.h" // PASS(), FAIL(), XPASS(), XFAIL()
 
 int
-main(int argc, char **argv, char **envp, char**appl)
+main(int argc, const char* argv[])
 {
   _dyld_launched_prebound();
   return EXIT_SUCCESS;
