@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2005-2007 Apple Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -28,11 +28,13 @@
 //
 // This builds an executable that is just big enough to force dyld to slide a bit
 //
+#if __arm__
+	#define ARRAY_SIZE  66582528
+#else
+	#define ARRAY_SIZE 335400000
+#endif
 
-#define ARRAY_SIZE 301800000
-
-int bigarray1[ARRAY_SIZE];
-int bigarray2[ARRAY_SIZE];
+//int bigarray1[ARRAY_SIZE];
 
 int
 main()
